@@ -2,7 +2,7 @@ from typing import Optional
 import numpy as np
 from numpy.typing import NDArray
 
-def jam(z: NDArray) -> NDArray:
+def jam(z: NDArray, real: bool = False) -> NDArray:
     """JAM activation function.
 
     A specialized activation function that uses the imaginary part of the sigmoid
@@ -19,10 +19,14 @@ def jam(z: NDArray) -> NDArray:
         >>> jam(x)
         array([1., 0., 1., 0.])
     """
-    s = 1 / (1 + np.exp(-z))
-    return (s.imag > 0).astype(np.float64)
+    if real:
+        s = 1 / (1 + np.exp(-z.real))
+        return (s > 0.5).astype(np.float64)
+    else:
+        s = 1 / (1 + np.exp(-z))
+        return (s.imag > 0).astype(np.float64)
 
-def jam_derivative(z: NDArray, grad_output: NDArray) -> NDArray:
+def jam_derivative(z: NDArray, grad_output: NDArray, real: bool = False) -> NDArray:
     """Derivative of the JAM activation function.
 
     Computes the gradient for backpropagation through JAM activation.
@@ -34,12 +38,17 @@ def jam_derivative(z: NDArray, grad_output: NDArray) -> NDArray:
     Returns:
         Gradient tensor for backpropagation
     """
-    s = 1 / (1 + np.exp(-z))
-    ds = s * (1 - s)
-    return grad_output * (ds.real + 1j * ds.imag)
+    if real:
+        s = 1 / (1 + np.exp(-z.real))
+        ds = s * (1 - s)
+        return grad_output * ds
+    else:
+        s = 1 / (1 + np.exp(-z))
+        ds = s * (1 - s)
+        return grad_output * (ds.real + 1j * ds.imag)
 
 
-def complex_relu(z: NDArray) -> NDArray:
+def complex_relu(z: NDArray, real: bool = False) -> NDArray:
     """Complex-valued ReLU activation function.
 
     Applies ReLU separately to real and imaginary parts:
@@ -56,9 +65,12 @@ def complex_relu(z: NDArray) -> NDArray:
         >>> complex_relu(x)
         array([1.+1.j, 0.+0.j])
     """
-    return np.maximum(z.real, 0) + 1j * np.maximum(z.imag, 0)
+    if real:
+        return np.maximum(z, 0)
+    else:
+        return np.maximum(z.real, 0) + 1j * np.maximum(z.imag, 0)
 
-def complex_relu_backward(z: NDArray, grad_output: NDArray) -> NDArray:
+def complex_relu_backward(z: NDArray, grad_output: NDArray, real: bool = False) -> NDArray:
     """Derivative of complex-valued ReLU.
 
     Computes gradients for backpropagation through complex ReLU.
@@ -70,14 +82,19 @@ def complex_relu_backward(z: NDArray, grad_output: NDArray) -> NDArray:
     Returns:
         Gradient tensor for backpropagation
     """
-    grad_real = grad_output.real * (z.real > 0)
-    grad_imag = grad_output.imag * (z.imag > 0)
-    return grad_real + 1j * grad_imag
+    if real:
+        return grad_output * (z > 0)
+    else:
+        grad_real = grad_output.real * (z.real > 0)
+        grad_imag = grad_output.imag * (z.imag > 0)
+        return grad_real + 1j * grad_imag
 
 # Additional complex activation functions
 
 # Separable (real/imag) sigmoid
-def complex_sigmoid(z, fully_complex=False):
+def complex_sigmoid(z, real: bool = False, fully_complex=False):
+    if real:
+        return 1 / (1 + np.exp(-z))
     if not fully_complex:
         s_real = 1 / (1 + np.exp(-z.real))
         s_imag = 1 / (1 + np.exp(-z.imag))
@@ -85,7 +102,10 @@ def complex_sigmoid(z, fully_complex=False):
     else:
         return 1 / (1 + np.exp(-z))
 
-def complex_sigmoid_backward(z, grad_output, fully_complex=False):
+def complex_sigmoid_backward(z, grad_output, real: bool = False, fully_complex=False):
+    if real:
+        s = 1 / (1 + np.exp(-z))
+        return grad_output * s * (1 - s)
     if not fully_complex:
         s_real = 1 / (1 + np.exp(-z.real))
         s_imag = 1 / (1 + np.exp(-z.imag))
@@ -98,7 +118,9 @@ def complex_sigmoid_backward(z, grad_output, fully_complex=False):
 
 
 # Separable (real/imag) tanh
-def complex_tanh(z, fully_complex=False):
+def complex_tanh(z, real: bool = False, fully_complex=False):
+    if real:
+        return np.tanh(z)
     if not fully_complex:
         t_real = np.tanh(z.real)
         t_imag = np.tanh(z.imag)
@@ -106,7 +128,10 @@ def complex_tanh(z, fully_complex=False):
     else:
         return np.tanh(z)
 
-def complex_tanh_backward(z, grad_output, fully_complex=False):
+def complex_tanh_backward(z, grad_output, real: bool = False, fully_complex=False):
+    if real:
+        t = np.tanh(z)
+        return grad_output * (1 - t ** 2)
     if not fully_complex:
         t_real = np.tanh(z.real)
         t_imag = np.tanh(z.imag)
